/*
 * libsysactivity
 * http://sourceforge.net/projects/libsysactivity/
 * Copyright (c) 2009 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <errno.h>
#include <string.h>
#include <inttypes.h>
#include <stdio.h>

#include "libsysactivity.h"

__thread FILE* file_net_dev;

int sa_open_network() {
	file_net_dev = fopen("/proc/net/dev", "r");
	if (file_net_dev == NULL)
		return EIO;
	return 0;
}

int sa_close_network() {
	if (file_net_dev != NULL)
		fclose(file_net_dev);
	return 0;
}

int sa_count_network_interfaces(uint16_t* number) {
	if (number == NULL)
		return EINVAL;
	if (fflush(file_net_dev) != 0)
		return EIO;
	if (fseek(file_net_dev, 200, SEEK_SET))
		return EIO;

	char line_buffer[256];
	*number = 0;
	while (fgets(line_buffer, sizeof line_buffer, file_net_dev)) {
		(*number)++;
	}
	return 0;
}

int sa_get_network_interface(char* name, struct sa_network_interface* dst) {
	if (name == NULL || dst == NULL)
		return EINVAL;

	if (fflush(file_net_dev) != 0)
		return EIO;
	if (fseek(file_net_dev, 200, SEEK_SET))
		return EIO;

	char line_buffer[256];
	char tmp_name[sizeof dst->name];
	while (fgets(line_buffer, sizeof line_buffer, file_net_dev)) {
		*strrchr(line_buffer, ':') = ' ';
		if (sscanf(line_buffer, "%63s ", tmp_name) != 1)
			return ENOSYS;
		if (strncmp(tmp_name, name, sizeof dst->name) != 0)
			continue;
		if (sscanf(
				line_buffer,
				"%63s %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %*s %"PRIu64" %*s %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %*s %*s %"PRIu64"",
				dst->name, &dst->received_bytes, &dst->received_packets, &dst->received_errors,
				&dst->received_drop, &dst->received_fifo, &dst->received_compressed,
				&dst->sent_bytes, &dst->sent_packets, &dst->sent_errors, &dst->sent_drop,
				&dst->sent_fifo, &dst->sent_compressed) != 13)
			return ENOSYS;

		return 0;
	}
	return ENODEV;
}

int sa_get_network_interfaces(struct sa_network_interface* dst, uint16_t dst_size,
		uint16_t* written) {
	if (dst == NULL || dst_size == 0 || written == NULL)
		return EINVAL;

	if (fflush(file_net_dev) != 0)
		return EIO;
	if (fseek(file_net_dev, 200, SEEK_SET))
		return EIO;

	*written = 0;
	int i;
	char line_buffer[256];
	for (i = 0; i < dst_size; i++) {
		if (fgets(line_buffer, sizeof line_buffer, file_net_dev) == NULL)
			return ENODEV;
		*strrchr(line_buffer, ':') = ' ';
		if (sscanf(
				line_buffer,
				"%63s %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %*s %"PRIu64" %*s %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %*s %*s %"PRIu64"",
				dst[i].name, &dst[i].received_bytes, &dst[i].received_packets,
				&dst[i].received_errors, &dst[i].received_drop, &dst[i].received_fifo,
				&dst[i].received_compressed, &dst[i].sent_bytes, &dst[i].sent_packets,
				&dst[i].sent_errors, &dst[i].sent_drop, &dst[i].sent_fifo, &dst[i].sent_compressed)
				!= 13)
			return ENOSYS;

		(*written)++;
	}
	return 0;
}
